"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const glob_1 = __importDefault(require("glob"));
const util_1 = __importDefault(require("util"));
// Convert fs.readFile into Promise version of same
const readFile = util_1.default.promisify(fs_1.default.readFile);
const globPromise = util_1.default.promisify(glob_1.default);
const tree_sitter_1 = __importDefault(require("tree-sitter"));
const tree_sitter_elm_1 = __importDefault(require("tree-sitter-elm"));
const vscode_uri_1 = __importDefault(require("vscode-uri"));
const utils = __importStar(require("../util/elmUtils"));
class ASTProvider {
    constructor(connection, forest, elmWorkspace, imports) {
        this.initializeWorkspace = () => __awaiter(this, void 0, void 0, function* () {
            try {
                const path = this.elmWorkspace.fsPath + "elm.json";
                this.connection.console.info("Reading elm.json from " + path);
                // Find elm files and feed them to tree sitter
                const elmJson = require(path);
                const type = elmJson.type;
                const elmFolders = [];
                let elmVersion = "";
                if (type === "application") {
                    elmVersion = elmJson["elm-version"];
                    const sourceDirs = elmJson["source-directories"];
                    sourceDirs.forEach((folder) => __awaiter(this, void 0, void 0, function* () {
                        elmFolders.push({
                            path: this.elmWorkspace.fsPath + folder,
                            writeable: true,
                        });
                    }));
                }
                else {
                    // Todo find a better way to do this
                    elmVersion = elmJson["elm-version"];
                    if (elmVersion.indexOf(" ") !== -1) {
                        elmVersion = elmVersion.substring(0, elmVersion.indexOf(" "));
                    }
                    elmFolders.push({
                        path: this.elmWorkspace.fsPath + "src",
                        writeable: true,
                    });
                }
                elmFolders.push({
                    path: this.elmWorkspace.fsPath + "tests",
                    writeable: true,
                });
                this.connection.console.info(elmFolders.length + " source-dirs found");
                const elmHome = this.findElmHome();
                // TODO find a way to detect this
                const packagesRoot = `${elmHome}/${elmVersion}/package/`;
                const dependencies = type === "application"
                    ? Object.assign({}, elmJson.dependencies.direct, elmJson["test-dependencies"].direct) : Object.assign({}, elmJson.dependencies, elmJson["test-dependencies"]);
                for (const key in dependencies) {
                    if (dependencies.hasOwnProperty(key)) {
                        const maintainer = key.substring(0, key.indexOf("/"));
                        const packageName = key.substring(key.indexOf("/") + 1, key.length);
                        // We should probably parse the elm json of a dependency, at some point down the line
                        const pathToPackage = type === "application"
                            ? `${packagesRoot}${maintainer}/${packageName}/${dependencies[key]}/src`
                            : `${packagesRoot}${maintainer}/${packageName}/${dependencies[key].substring(0, elmVersion.indexOf(" "))}`;
                        elmFolders.push({ path: pathToPackage, writeable: false });
                    }
                }
                const elmFilePaths = yield this.findElmFilesInFolders(elmFolders);
                this.connection.console.info("Found " +
                    elmFilePaths.length.toString() +
                    " files to add to the project");
                for (const filePath of elmFilePaths) {
                    this.connection.console.info("Adding " + filePath.path.toString());
                    const fileContent = yield readFile(filePath.path.toString(), "utf8");
                    let tree;
                    tree = this.parser.parse(fileContent);
                    this.forest.setTree(vscode_uri_1.default.file(filePath.path).toString(), filePath.writeable, true, tree);
                }
                this.forest.treeIndex.forEach(item => {
                    this.connection.console.info("Adding imports " + item.uri.toString());
                    this.imports.updateImports(item.uri, item.tree, this.forest);
                });
                this.connection.console.info("Done parsing all files.");
            }
            catch (error) {
                this.connection.console.info(error.toString());
            }
        });
        this.connection = connection;
        this.forest = forest;
        this.elmWorkspace = elmWorkspace;
        this.imports = imports;
        this.parser = new tree_sitter_1.default();
        try {
            this.parser.setLanguage(tree_sitter_elm_1.default);
        }
        catch (error) {
            this.connection.console.info(error.toString());
        }
        this.initializeWorkspace();
    }
    findElmHome() {
        const elmHomeVar = process.env.ELM_HOME;
        if (elmHomeVar) {
            return elmHomeVar;
        }
        const homedir = require("os").homedir();
        if (utils.isWindows) {
            return homedir + "/AppData/Roaming/elm";
        }
        else {
            return homedir + "/.elm";
        }
    }
    findElmFilesInFolders(elmFolders) {
        return __awaiter(this, void 0, void 0, function* () {
            let elmFilePaths = [];
            for (const element of elmFolders) {
                elmFilePaths = elmFilePaths.concat(yield this.findElmFilesInFolder(element));
            }
            return elmFilePaths;
        });
    }
    findElmFilesInFolder(element) {
        return __awaiter(this, void 0, void 0, function* () {
            return (yield globPromise(element.path + "/**/*.elm", {})).map((a) => {
                return { path: a, writeable: element.writeable };
            });
        });
    }
}
exports.ASTProvider = ASTProvider;
//# sourceMappingURL=astProvider.1.js.map