"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const vscode_uri_1 = require("vscode-uri");
const position_1 = require("../position");
class ASTProvider {
    constructor(connection, forest, events, imports, parser) {
        this.connection = connection;
        this.forest = forest;
        this.imports = imports;
        this.parser = parser;
        this.handleChangeTextDocument = (params) => __awaiter(this, void 0, void 0, function* () {
            this.connection.console.info(`Changed text document, going to parse it. ${params.textDocument.uri}`);
            const document = params.textDocument;
            let tree = this.forest.getTree(document.uri);
            if (tree === undefined) {
                const fileContent = fs_1.readFileSync(vscode_uri_1.URI.parse(document.uri).fsPath, "utf8");
                tree = this.parser.parse(fileContent);
            }
            for (const changeEvent of params.contentChanges) {
                if (changeEvent.range && changeEvent.rangeLength) {
                    // range is range of the change. end is exclusive
                    // rangeLength is length of text removed
                    // text is new text
                    const { range, rangeLength, text } = changeEvent;
                    const startIndex = range.start.line * range.start.character;
                    const oldEndIndex = startIndex + rangeLength - 1;
                    if (tree) {
                        tree.edit({
                            // end index for new version of text
                            newEndIndex: range.end.line * range.end.character - 1,
                            // position in new doc change ended
                            newEndPosition: position_1.Position.FROM_VS_POSITION(range.end).toTSPosition(),
                            // end index for old version of text
                            oldEndIndex,
                            // position in old doc change ended.
                            oldEndPosition: this.computeEndPosition(startIndex, oldEndIndex, tree),
                            // index in old doc the change started
                            startIndex,
                            // position in old doc change started
                            startPosition: position_1.Position.FROM_VS_POSITION(range.start).toTSPosition(),
                        });
                        tree = this.parser.parse(text, tree);
                    }
                }
                else {
                    tree = this.parser.parse(changeEvent.text);
                }
            }
            if (tree) {
                this.forest.setTree(document.uri, true, true, tree);
                this.imports.updateImports(document.uri, tree, this.forest);
            }
        });
        this.computeEndPosition = (startIndex, endIndex, tree) => {
            const node = tree.rootNode.descendantForIndex(startIndex, endIndex);
            return node.endPosition;
        };
        events.on("change", this.handleChangeTextDocument);
    }
}
exports.ASTProvider = ASTProvider;
//# sourceMappingURL=astProvider.js.map