"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_languageserver_1 = require("vscode-languageserver");
const treeUtils_1 = require("../util/treeUtils");
class DefinitionProvider {
    constructor(connection, forest, imports) {
        this.connection = connection;
        this.forest = forest;
        this.imports = imports;
        this.handleDefinitionRequest = (param) => __awaiter(this, void 0, void 0, function* () {
            this.connection.console.info(`A definition was requested`);
            const tree = this.forest.getTree(param.textDocument.uri);
            if (tree) {
                const nodeAtPosition = tree.rootNode.namedDescendantForPosition({
                    column: param.position.character,
                    row: param.position.line,
                });
                const definitionNode = treeUtils_1.TreeUtils.findDefinitionNodeByReferencingNode(nodeAtPosition, param.textDocument.uri, tree, this.imports);
                if (definitionNode) {
                    return this.createLocationFromDefinition(definitionNode.node, definitionNode.uri);
                }
            }
        });
        this.connection.onDefinition(this.handleDefinitionRequest);
    }
    createLocationFromDefinition(definitionNode, uri) {
        if (definitionNode) {
            return vscode_languageserver_1.Location.create(uri, vscode_languageserver_1.Range.create(vscode_languageserver_1.Position.create(definitionNode.startPosition.row, definitionNode.startPosition.column), vscode_languageserver_1.Position.create(definitionNode.endPosition.row, definitionNode.endPosition.column)));
        }
    }
}
exports.DefinitionProvider = DefinitionProvider;
//# sourceMappingURL=definitionProvider.js.map