"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_uri_1 = require("vscode-uri");
class DiagnosticsProvider {
    constructor(connection, elmWorkspaceFolder, events, elmAnalyse, elmMake) {
        this.connection = connection;
        this.elmWorkspaceFolder = elmWorkspaceFolder;
        this.events = events;
        this.getDiagnosticsOnSaveOrOpen = this.getDiagnosticsOnSaveOrOpen.bind(this);
        this.getDiagnosticsOnChange = this.getDiagnosticsOnChange.bind(this);
        this.newElmAnalyseDiagnostics = this.newElmAnalyseDiagnostics.bind(this);
        this.elmMakeDiagnostics = elmMake;
        this.elmAnalyseDiagnostics = elmAnalyse;
        this.currentDiagnostics = {
            elmAnalyse: new Map(),
            elmMake: new Map(),
            elmTest: new Map(),
        };
        this.events.on("open", this.getDiagnosticsOnSaveOrOpen);
        this.events.on("change", this.getDiagnosticsOnChange);
        this.events.on("save", this.getDiagnosticsOnSaveOrOpen);
        this.elmAnalyseDiagnostics.on("new-diagnostics", this.newElmAnalyseDiagnostics);
    }
    newElmAnalyseDiagnostics(diagnostics) {
        this.currentDiagnostics.elmAnalyse = diagnostics;
        this.sendDiagnostics();
    }
    sendDiagnostics() {
        const allDiagnostics = new Map();
        for (const [uri, diagnostics] of this.currentDiagnostics.elmMake) {
            allDiagnostics.set(uri, diagnostics);
        }
        for (const [uri, diagnostics] of this.currentDiagnostics.elmTest) {
            const currentDiagnostics = allDiagnostics.get(uri) || [];
            if (currentDiagnostics.length === 0) {
                allDiagnostics.set(uri, diagnostics);
            }
        }
        for (const [uri, diagnostics] of this.currentDiagnostics.elmAnalyse) {
            const currentDiagnostics = allDiagnostics.get(uri) || [];
            if (currentDiagnostics.length === 0) {
                allDiagnostics.set(uri, diagnostics);
            }
        }
        for (const [uri, diagnostics] of allDiagnostics) {
            this.connection.sendDiagnostics({ uri, diagnostics });
        }
    }
    getDiagnosticsOnChange(document) {
        return __awaiter(this, void 0, void 0, function* () {
            this.connection.console.info("Diagnostics were requested due to a file change");
            this.getDiagnostics(document, false);
        });
    }
    getDiagnosticsOnSaveOrOpen(document) {
        return __awaiter(this, void 0, void 0, function* () {
            this.connection.console.info("Diagnostics were requested due to a file open or save");
            this.getDiagnostics(document, true);
        });
    }
    getDiagnostics(document, isSaveOrOpen) {
        return __awaiter(this, void 0, void 0, function* () {
            const uri = vscode_uri_1.URI.parse(document.uri);
            if (uri.toString().startsWith(this.elmWorkspaceFolder.toString())) {
                const text = document.getText();
                if (isSaveOrOpen) {
                    this.currentDiagnostics.elmMake = yield this.elmMakeDiagnostics.createDiagnostics(uri);
                }
                const elmMakeDiagnosticsForCurrentFile = this.currentDiagnostics.elmMake.get(uri.toString());
                if (elmMakeDiagnosticsForCurrentFile &&
                    elmMakeDiagnosticsForCurrentFile.length === 0) {
                    this.elmAnalyseDiagnostics.updateFile(uri, text);
                }
                this.sendDiagnostics();
            }
        });
    }
}
exports.DiagnosticsProvider = DiagnosticsProvider;
//# sourceMappingURL=diagnosticsProvider.js.map