/// <reference types="node" />
import { EventEmitter } from "events";
import { CodeAction, CodeActionParams, Diagnostic, ExecuteCommandParams, IConnection } from "vscode-languageserver";
import { URI } from "vscode-uri";
import { Settings } from "../../util/settings";
import { TextDocumentEvents } from "../../util/textDocumentEvents";
import { DocumentFormattingProvider } from "../documentFormatingProvider";
export declare const CODE_ACTION_ELM_ANALYSE: string;
export declare const CODE_ACTION_ELM_ANALYSE_FIX_ALL: string;
export interface IElmAnalyseEvents {
    on(event: "new-report", diagnostics: Map<string, Diagnostic[]>): this;
}
export declare class ElmAnalyseDiagnostics extends EventEmitter {
    private connection;
    private elmWorkspace;
    private elmAnalyse;
    private diagnostics;
    private filesWithDiagnostics;
    private events;
    private settings;
    private formattingProvider;
    constructor(connection: IConnection, elmWorkspace: URI, events: TextDocumentEvents, settings: Settings, formattingProvider: DocumentFormattingProvider);
    updateFile(uri: URI, text?: string): void;
    onCodeAction(params: CodeActionParams): CodeAction[];
    onExecuteCommand(params: ExecuteCommandParams): Promise<import("vscode-languageserver").ApplyWorkspaceEditResponse | undefined>;
    private fixableDiagnostics;
    /**
     * If a diagnosticId is provided it will fix the single issue, if no
     * id is provided it will fix the entire file.
     */
    private fixer;
    private getFixEdits;
    private createEdits;
    private setupElmAnalyse;
    private onNewReport;
    private isFixable;
    private messageToDiagnostic;
}
