"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_languageserver_1 = require("vscode-languageserver");
const vscode_uri_1 = require("vscode-uri");
class ElmDiagnosticsHelper {
    static issuesToDiagnosticMap(issues, elmWorkspaceFolder) {
        return issues.reduce((acc, issue) => {
            const uri = this.getUriFromIssue(issue, elmWorkspaceFolder);
            const diagnostic = this.elmMakeIssueToDiagnostic(issue);
            const arr = acc.get(uri) || [];
            arr.push(diagnostic);
            acc.set(uri, arr);
            return acc;
        }, new Map());
    }
    static severityStringToDiagnosticSeverity(severity) {
        switch (severity) {
            case "error":
                return vscode_languageserver_1.DiagnosticSeverity.Error;
            case "warning":
                return vscode_languageserver_1.DiagnosticSeverity.Warning;
            default:
                return vscode_languageserver_1.DiagnosticSeverity.Error;
        }
    }
    static getUriFromIssue(issue, elmWorkspaceFolder) {
        if (issue.file.startsWith(".")) {
            return vscode_uri_1.URI.file(elmWorkspaceFolder + issue.file.slice(1)).toString();
        }
        return vscode_uri_1.URI.file(elmWorkspaceFolder.fsPath + issue.file).toString();
    }
    static elmMakeIssueToDiagnostic(issue) {
        const lineRange = vscode_languageserver_1.Range.create(issue.region.start.line - 1, issue.region.start.column - 1, issue.region.end.line - 1, issue.region.end.column - 1);
        const messagePrefix = issue.overview ? `${issue.overview} - ` : "";
        return vscode_languageserver_1.Diagnostic.create(lineRange, `${messagePrefix}${issue.details.replace(/\[\d+m/g, "")}`, this.severityStringToDiagnosticSeverity(issue.type), undefined, "Elm");
    }
}
exports.ElmDiagnosticsHelper = ElmDiagnosticsHelper;
//# sourceMappingURL=elmDiagnosticsHelper.js.map