"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const cp = __importStar(require("child_process"));
const crypto = __importStar(require("crypto"));
const path = __importStar(require("path"));
const readline = __importStar(require("readline"));
const vscode_languageserver_1 = require("vscode-languageserver");
const utils = __importStar(require("../../util/elmUtils"));
const elmDiagnosticsHelper_1 = require("./elmDiagnosticsHelper");
const ELM_MAKE = "Elm";
const RANDOM_ID = crypto.randomBytes(16).toString("hex");
exports.CODE_ACTION_ELM_MAKE = `elmLS.elmMakeFixer-${RANDOM_ID}`;
class ElmMakeDiagnostics {
    constructor(connection, elmWorkspaceFolder, settings) {
        this.connection = connection;
        this.elmWorkspaceFolder = elmWorkspaceFolder;
        this.settings = settings;
        this.createDiagnostics = (filePath) => __awaiter(this, void 0, void 0, function* () {
            return yield this.checkForErrors(this.connection, this.elmWorkspaceFolder.fsPath, filePath.fsPath).then(issues => {
                return issues.length === 0
                    ? new Map([[filePath.toString(), []]])
                    : elmDiagnosticsHelper_1.ElmDiagnosticsHelper.issuesToDiagnosticMap(issues, this.elmWorkspaceFolder);
            });
        });
        this.connection = connection;
        this.elmWorkspaceFolder = elmWorkspaceFolder;
        this.settings = settings;
    }
    onCodeAction(params) {
        const { uri } = params.textDocument;
        const elmMakeDiagnostics = this.filterElmMakeDiagnostics(params.context.diagnostics);
        return this.convertDiagnosticsToCodeActions(elmMakeDiagnostics, uri);
    }
    convertDiagnosticsToCodeActions(diagnostics, uri) {
        const result = [];
        diagnostics.forEach(diagnostic => {
            if (diagnostic.message.startsWith("NAMING ERROR")) {
                // Offer the name suggestions from elm make to our users
                const regex = /^\s{4}#(.*)#$/gm;
                let matches;
                // tslint:disable-next-line: no-conditional-assignment
                while ((matches = regex.exec(diagnostic.message)) !== null) {
                    // This is necessary to avoid infinite loops with zero-width matches
                    if (matches.index === regex.lastIndex) {
                        regex.lastIndex++;
                    }
                    matches
                        .filter((_, groupIndex) => groupIndex === 1)
                        .forEach((match, _) => {
                        result.push(this.createQuickFix(uri, match, diagnostic));
                    });
                }
            }
            else if (diagnostic.message.startsWith("MODULE NAME MISMATCH")) {
                // Offer the name suggestions from elm make to our users
                const regex = /# -> #(.*)#$/gm;
                const matches = regex.exec(diagnostic.message);
                if (matches !== null) {
                    result.push(this.createQuickFix(uri, matches[1], diagnostic));
                }
            }
        });
        return result;
    }
    createQuickFix(uri, replaceWith, diagnostic) {
        const map = {};
        if (!map[uri]) {
            map[uri] = [];
        }
        map[uri].push(vscode_languageserver_1.TextEdit.replace(diagnostic.range, replaceWith));
        return {
            diagnostics: [diagnostic],
            edit: { changes: map },
            kind: vscode_languageserver_1.CodeActionKind.QuickFix,
            title: replaceWith,
        };
    }
    filterElmMakeDiagnostics(diagnostics) {
        return diagnostics.filter(diagnostic => diagnostic.source === ELM_MAKE);
    }
    checkForErrors(connection, cwd, filename) {
        return __awaiter(this, void 0, void 0, function* () {
            const settings = yield this.settings.getSettings(connection);
            return new Promise((resolve, reject) => {
                let relativePathToFile = path.relative(cwd, filename);
                if (utils.isWindows) {
                    relativePathToFile = `"${relativePathToFile}"`;
                }
                const argsMake = [
                    "make",
                    relativePathToFile,
                    "--report",
                    "json",
                    "--output",
                    "/dev/null",
                ];
                const argsTest = [
                    "make",
                    relativePathToFile,
                    "--report",
                    "json",
                    "--output",
                    "/dev/null",
                ];
                const makeCommand = settings.elmPath;
                const testCommand = settings.elmTestPath;
                const isTestFile = utils.isTestFile(filename, cwd);
                const args = isTestFile ? argsTest : argsMake;
                const testOrMakeCommand = isTestFile ? testCommand : makeCommand;
                let make;
                if (utils.isWindows) {
                    make = cp.exec(`${testOrMakeCommand} ${args.join(" ")}`, { cwd });
                }
                else {
                    make = cp.spawn(testOrMakeCommand, args, { cwd });
                }
                if (!make.stderr) {
                    return;
                }
                const errorLinesFromElmMake = readline.createInterface({
                    input: make.stderr,
                });
                const lines = [];
                errorLinesFromElmMake.on("line", (line) => {
                    const errorObject = JSON.parse(line);
                    if (errorObject.type === "compile-errors") {
                        errorObject.errors.forEach((error) => {
                            const problems = error.problems.map((problem) => ({
                                details: problem.message
                                    .map((message) => typeof message === "string" ? message : `#${message.string}#`)
                                    .join(""),
                                file: path.isAbsolute(error.path)
                                    ? path.relative(cwd, error.path)
                                    : error.path,
                                overview: problem.title,
                                region: problem.region,
                                subregion: "",
                                tag: "error",
                                type: "error",
                            }));
                            lines.push(...problems);
                        });
                    }
                    else if (errorObject.type === "error") {
                        const problem = {
                            details: errorObject.message
                                .map((message) => typeof message === "string" ? message : message.string)
                                .join(""),
                            file: errorObject.path,
                            overview: errorObject.title,
                            region: {
                                end: {
                                    column: 1,
                                    line: 1,
                                },
                                start: {
                                    column: 1,
                                    line: 1,
                                },
                            },
                            subregion: "",
                            tag: "error",
                            type: "error",
                        };
                        lines.push(problem);
                    }
                });
                make.on("error", (err) => {
                    errorLinesFromElmMake.close();
                    if (err && err.code === "ENOENT") {
                        connection.window.showErrorMessage(isTestFile
                            ? "'elm-test' is not available. Install Elm via 'npm install -g elm-test'."
                            : "The 'elm' compiler is not available. Install Elm via 'npm install -g elm'.");
                        resolve([]);
                    }
                    else {
                        reject(err);
                    }
                });
                make.on("close", (code, signal) => {
                    errorLinesFromElmMake.close();
                    resolve(lines);
                });
            });
        });
    }
}
exports.ElmMakeDiagnostics = ElmMakeDiagnostics;
//# sourceMappingURL=elmMakeDiagnostics.js.map