"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const cp = __importStar(require("child_process"));
const path = __importStar(require("path"));
const readline = __importStar(require("readline"));
const utils = __importStar(require("../../util/elmUtils"));
const elmDiagnosticsHelper_1 = require("./elmDiagnosticsHelper");
class ElmTestDiagnostics {
    constructor(connection, elmWorkspaceFolder, settings) {
        this.connection = connection;
        this.elmWorkspaceFolder = elmWorkspaceFolder;
        this.settings = settings;
        this.createDiagnostics = (filePath) => __awaiter(this, void 0, void 0, function* () {
            return yield this.checkForErrors(this.connection, this.elmWorkspaceFolder.fsPath, filePath.fsPath).then(issues => {
                if (issues && issues.length > 0) {
                    return elmDiagnosticsHelper_1.ElmDiagnosticsHelper.issuesToDiagnosticMap(issues, this.elmWorkspaceFolder);
                }
                else {
                    return new Map([[filePath.toString(), []]]);
                }
            });
        });
        this.connection = connection;
        this.elmWorkspaceFolder = elmWorkspaceFolder;
        this.settings = settings;
    }
    checkForErrors(connection, cwd, filename) {
        return __awaiter(this, void 0, void 0, function* () {
            const settings = yield this.settings.getSettings(connection);
            return new Promise((resolve, reject) => {
                const isTestFile = utils.isTestFile(filename, cwd);
                if (isTestFile) {
                    const testCommand = settings.elmTestPath;
                    let make;
                    if (utils.isWindows) {
                        filename = `"${filename}"`;
                    }
                    const argsTest = [path.relative(cwd, filename), "--report", "json"];
                    if (utils.isWindows) {
                        make = cp.exec(`${testCommand} ${argsTest.join(" ")}`, { cwd });
                    }
                    else {
                        make = cp.spawn(testCommand, argsTest, { cwd });
                    }
                    const elmTestResult = this.readElmTestResult(make, filename);
                    if (elmTestResult) {
                        const { errorLinesFromElmTest: errorLinesFromElmTest, lines: lines, } = elmTestResult;
                        make.on("error", (err) => {
                            errorLinesFromElmTest.close();
                            if (err && err.code === "ENOENT") {
                                connection.window.showErrorMessage("'elm-test' is not available. Install Elm via 'npm install -g elm-test'.");
                                resolve([]);
                            }
                            else {
                                reject(err);
                            }
                        });
                        make.on("close", (code, signal) => {
                            errorLinesFromElmTest.close();
                            resolve(lines);
                        });
                    }
                }
                else {
                    resolve();
                }
            });
        });
    }
    readElmTestResult(make, filename) {
        if (!make.stdout) {
            return;
        }
        const errorLinesFromElmMake = readline.createInterface({
            input: make.stdout,
        });
        const lines = [];
        errorLinesFromElmMake.on("line", (line) => {
            const errorObject = JSON.parse(line);
            if (errorObject.event === "testCompleted" &&
                errorObject.status === "fail") {
                errorObject.failures.forEach((failure) => {
                    lines.push({
                        details: `Comparison: ${failure.reason.data.comparison}\n Expected: ${failure.reason.data.expected}\n Actual: ${failure.reason.data.actual}\n`,
                        file: filename,
                        overview: failure.reason.type,
                        region: {
                            end: {
                                column: 1,
                                line: 1,
                            },
                            start: {
                                column: 1,
                                line: 1,
                            },
                        },
                        subregion: "",
                        tag: "test",
                        type: "error",
                    });
                });
            }
        });
        return { errorLinesFromElmTest: errorLinesFromElmMake, lines };
    }
}
exports.ElmTestDiagnostics = ElmTestDiagnostics;
//# sourceMappingURL=elmTestDiagnostics.js.map