/// <reference types="node" />
import { IConnection } from "vscode-languageserver";
import { URI } from "vscode-uri";
import { Settings } from "./settings";
export declare const isWindows: boolean;
/** Options for execCmd */
export interface IExecCmdOptions {
    /** Any arguments */
    cmdArguments?: string[];
    /** Shows a message if an error occurs (in particular the command not being */
    showMessageOnError?: boolean;
    /** Called after the process successfully starts */
    onStart?: () => void;
    /** Called when data is sent to stdout */
    onStdout?: (data: string) => void;
    /** Called when data is sent to stderr */
    onStderr?: (data: string) => void;
    /** Called after the command (successfully or unsuccessfully) exits */
    onExit?: () => void;
    /** Text to add when command is not found (maybe helping how to install) */
    notFoundText?: string;
}
/** Type returned from execCmd. Is a promise for when the command completes
 *  and also a wrapper to access ChildProcess-like methods.
 */
export interface IExecutingCmd extends Promise<{
    stdout: string;
    stderr: string;
}> {
    /** The process's stdin */
    stdin: NodeJS.WritableStream;
    /** End the process */
    kill(): void;
    /** Is the process running */
    isRunning: boolean;
}
/** Executes a command. Shows an error message if the command isn't found */
export declare function execCmd(cmd: string, options: IExecCmdOptions | undefined, elmRootPath: URI, connection: IConnection): IExecutingCmd;
export declare function isTestFile(filename: string, rootPath: string): boolean;
export declare function getEmptyTypes(): {
    markdown: string;
    name: string;
    symbolKind: 10;
}[];
export declare function getElmVersion(settings: Settings, elmWorkspaceFolder: URI, connection: IConnection): Promise<string | undefined>;
