"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const prebuildInstall = __importStar(require("prebuild-install"));
function packageToGithubRepo(name) {
    return name === "tree-sitter" ? "node-tree-sitter" : name;
}
function downloadUrl(name, version, treeSitterRepo, runtime) {
    const repo = packageToGithubRepo(name);
    let urlBase = `https://github.com/tree-sitter/${repo}/releases/download/v${version}/`;
    if (!treeSitterRepo) {
        urlBase = `https://github.com/razzeee/${repo}/releases/download/v${version}/`;
    }
    const prebuild = `${name}-v${version}-${runtime}-v${process.versions.modules}-${process.platform}-${process.arch}.tar.gz`;
    return `${urlBase}${prebuild}`;
}
function fetchPrebuild(name, treeSitterRepo, console, runtime) {
    // tslint:disable-next-line: no-console
    console.info(`Fetching ${name}`);
    // Using require.resolve so that this works for both npm and yarn global installs, with
    // npm our node modules are in `<npm global dir/lib/elm-language-server/node_modules/`, but in yarn they
    // can be in a different location, like `<yarn global dir>/node_modules/tree-sitter`
    const pkgRoot = path.dirname(require.resolve(`${name}/package.json`));
    // tslint:disable-next-line non-literal-require
    const pkg = require(`${pkgRoot}/package.json`);
    const url = downloadUrl(pkg.name, pkg.version, treeSitterRepo, runtime);
    // tslint:disable-next-line: no-console
    console.info(`Downloading (or using local cache for) ${url}`);
    return new Promise((res, rej) => {
        // try to download+unpack the definition files only if they aren't already
        fs.access(`${pkgRoot}/build/Release`, fs.constants.F_OK, (accessErr) => {
            if (!accessErr) {
                return res();
            }
            if (accessErr.code !== "ENOENT") {
                return rej(accessErr);
            }
            prebuildInstall.download(url, { pkg, path: pkgRoot }, (err) => {
                err ? rej(err) : res();
            });
        });
    });
}
function rebuildTreeSitter(console, runtime) {
    return Promise.all([
        fetchPrebuild("tree-sitter", true, console, runtime),
        fetchPrebuild("tree-sitter-elm", false, console, runtime),
    ]);
}
exports.rebuildTreeSitter = rebuildTreeSitter;
//# sourceMappingURL=rebuilder.js.map