/// <reference types="node" />
import { EventEmitter } from "events";
import { TextDocument, TextDocumentContentChangeEvent } from "vscode-languageserver";
import { DocumentEvents } from "./documentEvents";
declare type DidChangeCallback = (document: TextDocument) => void;
declare type DidCloseCallback = (document: TextDocument) => void;
declare type DidOpenCallback = (document: TextDocument) => void;
declare type DidSaveCallback = (document: TextDocument) => void;
export interface ITextDocumentEvents {
    on(event: "change", listener: DidChangeCallback): this;
    on(event: "close", listener: DidCloseCallback): this;
    on(event: "open", listener: DidOpenCallback): this;
    on(event: "save", listener: DidSaveCallback): this;
}
interface IUpdatableDocument extends TextDocument {
    update(event: TextDocumentContentChangeEvent, version: number): void;
}
export declare class TextDocumentEvents extends EventEmitter implements ITextDocumentEvents {
    static isUpdatableDocument(value: TextDocument): value is IUpdatableDocument;
    private documents;
    constructor(events: DocumentEvents);
    /**
     * Returns the document for the given URI. Returns undefined if
     * the document is not managed by this instance.
     *
     * @param uri The text document's URI to retrieve.
     * @return the text document or `undefined`.
     */
    get(uri: string): TextDocument | undefined;
}
export {};
